
################################################################################
# Copyright (c) QinetiQ Plc 2003
#
# Licensed under the LGPL. For full license details see the LICENSE file.
################################################################################

"""
Initialisation support for hooking the GeoTypes into the psycopg typecast machinery.
"""

from _Point import Point
from _LineSeg import LineSeg
from _Path import Path
from _Polygon import Polygon
from _Circle import Circle
from _Box import Box

_class_map = {'point':   {'class': Point, 'oid': 600, 'name': 'POINT'},
              'lseg':    {'class': LineSeg, 'oid': 601, 'name': 'LINESEG'},
              'path':    {'class': Path,  'oid': 602, 'name': 'PATH'},
              'polygon': {'class': Polygon, 'oid': 604, 'name': 'POLYGON'},
              'circle':  {'class': Circle, 'oid': 718, 'name': 'CIRCLE'},
              'box':     {'class': Box, 'oid': 603, 'name': 'BOX'}}

def initialisePsycopgTypes(psycopg_module, subclass_map={}, connect_string=None):
    """
    Inform psycopg about the GeoType types.

    This ensures that when the 'bound' variable method of query generation is used
    the GeoTypes are automatically coverted into the right format for use in postgres
    queries.

    It also ensures that any results columns from queries that are of a geometric
    type are returned as instances of the GeoType classes.

    (arg psycopg_module) is the psycopg module itself. This is passes in as a parameter
    to ensure that the GeoTypes package can be used with psycopg is required. If it
    were imported in the module directly it work reduce the flexibility of the package.

    (arg subclass_map) is a dictionary of 'postgres_type':'GeoType class' pairs. It allows
    a caller to subclass the GeoType classes and ensure that the subclasses get hooked into
    the psycopg type machinery rather the plain GeoType classes. See the PsycopgInit_Test.py
    file for an example of its use. Default {}.

    (arg connect_string) is a postgres connection string of the form
    'dbname=schema_test user=postgres. If this is not None an attempt it made to create a
    test table from which to dynamically workout the type OIDS for the geometric types. If this
    is None as list of OID values that where correct when I tested it are used. I am not sure
    how oftern these OIDs change but if you have problems with typecast registration try
    passing a connect_string and see if it fixes it. Default None.
    
    """

    if subclass_map != {}:
        for override in subclass_map.keys():
            _class_map[override]['class'] = subclass_map[override]

    if connect_string != None:
        conn = psycopg_module.connect(connect_string)
        curs = conn.cursor()

        try:
            curs.execute("CREATE TABLE geotype_check_oids (point point, lseg lseg, path path, polygon polygon, circle circle, box box)")
        except:
            conn.rollback()
            curs.execute("DROP TABLE geotype_check_oids")
            curs.execute("CREATE TABLE geotype_check_oids (point point, lseg lseg, path path, polygon polygon, circle circle, box box)")
        conn.commit()

        curs.execute("SELECT point, lseg, path, polygon, circle, box FROM geotype_check_oids WHERE 0=1")

        for col in curs.description:
            _class_map[col[0]]['oid'] = col[1]

    for new_class in _class_map.values():
        psycopg_module.register_type(psycopg_module.new_type((new_class['oid'],), new_class['name'], new_class['class']))

    
    
        
        
    
